import { Component, Input, OnInit } from '@angular/core';
import { UntypedFormBuilder, UntypedFormGroup, Validators } from '@angular/forms';
import { Router } from '@angular/router';
import { User } from 'app/core/models/user';
import { BaseComponent } from 'app/shared/components/base/base.component';
import { BackendError } from 'app/shared/models/backend-error';
import { NotificationService } from 'app/shared/services/notification.service';
import { filter, flatMap, map, takeUntil } from 'rxjs/operators';
import { Course } from '../../models/course';
import { StaffGroup } from '../../models/staff-group';
import { CourseStaffMembership } from '../../models/course-staff-membership';
import { CourseService } from '../../services/course.service';
import { DialogAddStaffMembershipComponent } from '../../dialogs/dialog-add-staff-membership/dialog-add-staff-membership.component';
import { NbDialogService } from '@nebular/theme';

@Component({
  selector: 'ngx-course-staff-memberships-form',
  templateUrl: './course-staff-memberships-form.component.html',
  styleUrls: ['./course-staff-memberships-form.component.scss']
})
export class CourseStaffMembershipsFormComponent extends BaseComponent implements OnInit {
  @Input() course: Course;
  memberships: CourseStaffMembership[] = [];
  loading: boolean = true;
  StaffGroup = StaffGroup;

  tags: Set<string> = new Set<string>();

  error: BackendError;

  constructor(
    private courseService: CourseService,
    private router: Router,
    private dialogService: NbDialogService,
    private notificationService: NotificationService,
  ) {
    super();
  }

  get staffGroups(): StaffGroup[] {
    return <any>Object.keys(StaffGroup).filter(k => !isNaN(Number(k))).map(k => Number(k));
  }

  ngOnInit(): void {

    this.courseService.getStaff(this.course.id)
      .pipe(
        takeUntil(this.unsubscribe),
      )
      .subscribe(memberships => {
        this.memberships = memberships;
        this.loading = false;
      }, (error) => {
        if (BackendError.isBackendError(error)) {
          this.error = error;
          this.loading = false;
        } else {
          throw error;
        }
      });
  }


  remove(membership: CourseStaffMembership) {
    this.courseService.removeStaff(this.course.id, membership.user.id, membership.group)
      .pipe(takeUntil(this.unsubscribe))
      .subscribe(() => {
        let idx = this.memberships.indexOf(membership);
        if (idx != -1) {
          this.memberships.splice(idx, 1);
        }

        this.notificationService.showSuccess("Berechtigung erfolgreich entfernt");
      }, (error: any) => {
        if (BackendError.isBackendError(error)) {
          this.notificationService.showError(
            error.format(
              "Fehler bei Entfernung der Berechtigung",
            )
          );
        } else {
          throw error;
        }
      });
  }

  add() {
    this.dialogService.open(DialogAddStaffMembershipComponent, {
      context: { }
    }).onClose.pipe(
      takeUntil(this.unsubscribe),
      filter(result => result && result.user && result.group != null),
      flatMap(result => this.courseService.addStaff(this.course.id, result.user.id, result.group))
    ).subscribe(membership => {
      this.memberships.splice(0, 0, membership);
      this.notificationService.showSuccess("Berechtigung erfolgreich hinzugefügt");
    }, (error: any) => {
      if (BackendError.isBackendError(error)) {
        this.notificationService.showError(
          error.format(
            "Fehler bei Erstellung der Berechtigung",
          )
        );
      } else {
        throw error;
      }
    });
  }
}
