import { AfterViewInit, Component, Input, OnInit, QueryList, ViewChild, ViewChildren } from '@angular/core';
import { UntypedFormGroup, UntypedFormBuilder } from '@angular/forms';
import { Router, ActivatedRoute } from '@angular/router';
import { CodemirrorComponent } from '@ctrl/ngx-codemirror';
import { AppStateService } from 'app/core/services/app.state.service';
import { AuthService } from 'app/core/services/auth.service';
import { Exercise } from 'app/features/exercises/models/exercise';
import { FileRef } from 'app/features/tasks/models/file-ref';
import { FileType } from 'app/features/tasks/models/file-type';
import { SubmissionMode } from 'app/features/tasks/models/submission-mode';
import { TaskService } from 'app/features/tasks/services/task.service';
import { BaseComponent } from 'app/shared/components/base/base.component';
import { BackendError } from 'app/shared/models/backend-error';
import { NotificationService } from 'app/shared/services/notification.service';
import { of, forkJoin, Observable, combineLatest } from 'rxjs';
import { takeUntil, flatMap, take } from 'rxjs/operators';
import { Submission } from '../../../exercises/models/submission';
import { Task } from '../../../tasks/models/task';
import { ExerciseService } from '../../../exercises/services/exercise.service';
import { Client } from '../../models/client';
import { Solution } from '../../models/solution';
import { SolutionService, SubmitFileSolution, SubmitInlineSolution } from '../../services/solution.service';
import { SubatoPrincipal } from 'app/core/models/principal/subato-principal';
import { PublicConfig } from 'app/core/models/public-config';
import { TaskInstance } from 'app/features/exercises/models/task-instance';
import { CodemirrorHelper } from 'app/shared/helper/codemirror.helper';

@Component({
  selector: 'ngx-submit-solution-form',
  templateUrl: './submit-solution-form.component.html',
  styleUrls: ['./submit-solution-form.component.scss']
})
export class SubmitSolutionFormComponent extends BaseComponent implements OnInit, AfterViewInit {
  @ViewChildren(CodemirrorComponent) codemirrorComponents: QueryList<CodemirrorComponent>;

  @Input() task: Task;
  @Input() exercise: Exercise;
  @Input() taskInstance: TaskInstance;
  @Input() submission: Submission;

  loading: boolean = true;
  error: BackendError;
  submitError: BackendError;

  solutionForm: UntypedFormGroup;
  isUploadingSolution: boolean = false;
  templateFile: FileRef;
  templateContent: string;
  darkMode = false;
  principal: SubatoPrincipal;
  config: PublicConfig;

  SubmissionMode = SubmissionMode;

  get submissionModes(): SubmissionMode[] {
    let submissionModes = [];
    if(this.task.supportsFileSubmissionMode) {
      submissionModes.push(SubmissionMode.FILE);
    }

    if(this.task.supportsTextSubmissionMode) {
      submissionModes.push(SubmissionMode.TEXT);
    }

    return submissionModes;
  }

  constructor(
    private solutionService: SolutionService,
    private exerciseService: ExerciseService,
    private appStateService: AppStateService,
    private taskService: TaskService,
    private router: Router,
    private route: ActivatedRoute,
    private authService: AuthService,
    private notificationService: NotificationService,
    private fb: UntypedFormBuilder) {
    super();
    this.config = appStateService.config;
  }

  ngAfterViewInit(): void {
    this.codemirrorComponents.changes.subscribe(c => {
      this.refreshEditors();
    });
  }

  loadTemplate() {
    if(this.task.supportsTextSubmissionMode && this.templateFile) {
      return this.exercise != null 
      ? this.exerciseService.getFile(this.exercise.id, this.task.id, this.templateFile)
      : this.taskService.getFile(this.task.id, this.templateFile);
    } else {
      return of(<string>"");
    }
  }

  ngOnInit(): void {
    this.solutionForm = this.fb.group({
      file: [''],
      solutionContent: [''],
      comment: [''],
      lateSubmit: [false],
      selectedMode: [this.task.defaultSubmissionMode]
    });

    combineLatest(this.appStateService.darkMode$, this.authService.principal$)
      .pipe(takeUntil(this.unsubscribe))
      .subscribe(([darkMode, principal]) => {
        this.darkMode = darkMode;
        this.principal = principal;
      });

    let loadFiles = this.exercise != null 
    ? this.exerciseService.getTaskInstanceFiles(this.exercise.id, this.task.id)
    : this.taskService.getFiles(this.task.id);
    
    loadFiles.pipe(
      takeUntil(this.unsubscribe),
      flatMap(files => {
        this.templateFile = files.find(f => f.type == FileType.Template && f.file.src);
        return this.loadTemplate();
      })
    )
    .subscribe(template => {
      this.templateContent = template;
      let lastSolution = this.appStateService.getLastSolution(this.task, this.exercise);

      this.solutionForm.patchValue({ solutionContent: lastSolution != null ? lastSolution : template });
      this.loading = false;
    }, (error: any) => {
      if (BackendError.isBackendError(error)) {
        this.error = error;
        this.loading = false;
      } else {
        throw error;
      }
    })
  }

  public reset() {
    this.solutionForm.patchValue({ solutionContent: this.templateContent });
    this.appStateService.clearLastSolution(this.task, this.exercise);
  }

  get selectedMode(): SubmissionMode {
    return this.solutionForm.get('selectedMode').value;
  }

  handleFile(file: File) {
    this.solutionForm.patchValue({
      file: file
    });
  }

  submitSolution() {
    this.isUploadingSolution = true;
    this.submitError = null;
    
    let submitSolution: Observable<Solution>;

    if(this.selectedMode == SubmissionMode.TEXT) {
      let dto: SubmitInlineSolution = {
        exerciseId: this.exercise?.id,
        taskId: this.task.id,
        comment: this.solutionForm.get('comment').value,
        content: this.solutionForm.get('solutionContent').value,
        lateSubmit: this.solutionForm.get('lateSubmit').value
      };

      submitSolution = this.solutionService.submitInlineSolution(dto);
      this.appStateService.setLastSolution(dto.content, this.task, this.exercise);
    } else {
      let file = this.solutionForm.get('file').value;
      let dto: SubmitFileSolution = {
        exerciseId: this.exercise?.id,
        taskId: this.task.id,
        comment: this.solutionForm.get('comment').value,
        lateSubmit: this.solutionForm.get('lateSubmit').value,
        submittedWith: Client.Web
      };

      submitSolution = this.solutionService.submitFileSolution(dto, file);
    }
    
    submitSolution
      .pipe(
        takeUntil(this.unsubscribe),
        flatMap(solution => {
          return forkJoin(
            of(solution),
            this.authService.loadPrincipal()
          );
        })
      )
      .subscribe(([solution, _]) => {
        this.isUploadingSolution = false;
        this.solutionForm.reset();
        this.router.navigate(['/', 'solutions', solution.id], {
          state: {
            justSubmitted: true
          }
        });
      }, (error: any) => {
        this.isUploadingSolution = false;
        if (BackendError.isBackendError(error)) {
          this.submitError = error;
          this.notificationService.showError(
            error.format(
              "Fehler beim Abgeben der Lösungen" 
            )
          );
        } else {
          throw error;
        }
      });
  }

  refreshEditors() {
    this.codemirrorComponents.forEach(component => {
      CodemirrorHelper.waitForEditor(component)
        .pipe(take(1))
        .subscribe(() => component.codeMirror.refresh());
    });
  }
}
