import { Component, Input, OnInit } from '@angular/core';
import { UntypedFormBuilder, UntypedFormGroup, Validators } from '@angular/forms';
import { Router, ActivatedRoute } from '@angular/router';
import { FormComponent } from 'app/shared/components/base/form.component';
import { BackendError } from 'app/shared/models/backend-error';
import { NotificationService } from 'app/shared/services/notification.service';
import { Observable, of } from 'rxjs';
import { takeUntil, flatMap } from 'rxjs/operators';
import { CreateLanguageDto } from '../../dto/create-language.dto';
import { Language } from '../../models/language';
import { LanguageService } from '../../services/language.service';

@Component({
  selector: 'ngx-language-form',
  templateUrl: './language-form.component.html',
  styleUrls: ['./language-form.component.scss']
})
export class LanguageFormComponent extends FormComponent implements OnInit {
  saving: boolean = false;
  loading: boolean = true;
  languageForm: UntypedFormGroup;
  
  @Input() language: Language;
  
  constructor(
    private languageService: LanguageService,
    private router: Router,
    private route: ActivatedRoute,
    private notificationService: NotificationService,
    private fb: UntypedFormBuilder) {
    super();
  }

  createForm(language?: Language) {
    this.languageForm = this.fb.group({
      id: [language?.id ?? '', Validators.required],
      name: [language?.name ?? '', Validators.required],
    });
  }

  ngOnInit(): void {
    this.createForm(this.language);
  }

  cancel() {
    this.router.navigate(['/', 'language']);
  }

  save() {
    this.saving = true;
    
    let dto: CreateLanguageDto = {
      id: this.languageForm.get('id').value,
      name: this.languageForm.get('name').value,
    };

    if (this.language != null) {
      this.languageService.update(this.language.id, dto)
        .pipe(takeUntil(this.unsubscribe))
        .subscribe(term => {
          this.saving = false;

          this.notificationService.showSuccess("Sprache erfolgreich aktualisiert");
          this.router.navigate(['/', 'language']);
        }, (error: any) => {
          this.saving = false;

          if (BackendError.isBackendError(error)) {
            if (error.isValidation()) {
              this.errors = this.getValidationErrors(error);
            } 
  
            this.notificationService.showError(
              error.format(
                "Fehler bei Aktualisierung der Sprache",
              )
            );
          } else {
            throw error;
          }
        });
    } else {
      this.languageService.create(dto)
        .pipe(takeUntil(this.unsubscribe))
        .subscribe(lang => {
          this.saving = false;

          this.notificationService.showSuccess("Sprache erfolgreich erstellt");
          this.router.navigate(['/', 'language']);
        }, (error: any) => {
          this.saving = false;

          if (BackendError.isBackendError(error)) {
            this.notificationService.showError(
              error.format(
                "Fehler bei Erstellung der Sprache" 
              )
            );
          } else {
            throw error;
          }
        });
    }
  }
}
