package de.hsrm.sls.subato.auth;

import de.hsrm.sls.subato.auth.principal.PrincipalService;
import de.hsrm.sls.subato.auth.principal.SubatoPrincipal;
import de.hsrm.sls.subato.modules.program.DegreeProgramRepository;
import de.hsrm.sls.subato.shared.helper.KeycloakRoleMapper;
import de.hsrm.sls.subato.user.User;
import de.hsrm.sls.subato.user.UserRepository;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.cache.interceptor.KeyGenerator;
import org.springframework.context.annotation.Bean;
import org.springframework.security.oauth2.jwt.Jwt;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * Responsible for importing users authenticated by keycloak into the system, using the supplied token.
 *
 * Be careful: This implementation is not thread-safe, the method calling {@link #handleToken(KeycloakAuthenticationToken)}
 * should be declared as synchronized. Synchronization is not done in this class
 * because @Transactional does not work properly with synchronized
 */
@Service
@Transactional
public class KeycloakLoginHandler {
    @Autowired
    private PrincipalService principalService;

    @Autowired
    private UserRepository userRepository;

    @Autowired
    private DegreeProgramRepository degreeProgramRepository;

    private static Logger logger = LoggerFactory.getLogger(KeycloakLoginHandler.class);

    @Bean
    public KeyGenerator keycloakAuthKeyGenerator() {
        return (target, method, params) -> {
            var auth = (Jwt) params[0];
            var alias = getAlias(auth);
            if(alias != null) {
                return alias;
            }

            return getUsername(auth);
        };
    }

    /**
     * if not cached, the user should be imported or updated (if exists) from the token
     * otherwise the cached principal for that user is returned
     *
     * Caching may be seen as redundant with the one in {@link PrincipalService#loadUserByUsername(String)}.
     * However, user details (the principal) can be loaded without a token, so it is necessary to do that.
     * In integration tests (for example) the user is loaded directly (without a keycloak token).
     *
     * caching here will prevent the user from getting updated from the token on every request
     */
    @Cacheable(value = "currentUser", keyGenerator = "keycloakAuthKeyGenerator")
    public SubatoPrincipal handleToken(Jwt auth) {
        String alias = getAlias(auth);
        String username = alias;

        if(alias != null) {
            logger.debug(String.format("Logged in with alias user, skipping import/update..."));
        } else {
            username = getUsername(auth);
            var mail = getMail(auth);

            var user = userRepository.findByLoginNameIgnoreCase(username).orElse(null);
            if(user == null && mail != null) {
                logger.debug(String.format("User with loginName %s not found, trying email %s", username, mail));
                user = userRepository.findByMailIgnoreCase(getMail(auth)).orElse(null);
            }

            if (user == null) {
                logger.debug("User not found, importing now...");
                user = new User();
            }

            updateUserFromToken(user, auth);
        }

        return (SubatoPrincipal) principalService.loadUserByUsername(username);
    }

    private User updateUserFromToken(User user, Jwt auth) {

        user.setLoginName(getUsername(auth));
        user.setFirstName(auth.getClaim("given_name"));
        user.setLastName(auth.getClaim("family_name"));
        user.setMail(getMail(auth));
        user.setActive(true);
        user.setProgram(null);

        var posCode = (String) auth.getClaim("stg");
        if(posCode != null) {
            var program = degreeProgramRepository.findByPosCode(posCode);
            user.setProgram(program);
        } else {
            user.setProgram(null);
        }

        user.getRoles().clear();
        user = userRepository.save(user);

        Map<String, Object> resourceAccess = auth.getClaim("realm_access");
        List<String> plainRoles = (List<String>) resourceAccess.get("roles");

        var roles = plainRoles.stream().map(r -> KeycloakRoleMapper.getRole(r))
                .filter(r -> r != null)
                .collect(Collectors.toList());
        user.getRoles().addAll(roles);
        user = userRepository.save(user);
        logger.debug("Imported/updated user %s".formatted(user.getLoginName()));

        return user;
    }

    private String getUsername(Jwt auth) {
        return auth.getClaim("preferred_username");
    }

    private String getMail(Jwt auth) {
        return auth.getClaim("email");
    }

    private String getAlias(Jwt auth) {
        return auth.getClaim("alias");
    }
}
