package de.hsrm.sls.subato.courses.result;

import de.hsrm.sls.subato.auth.policy.exercises.ExercisesAction;
import de.hsrm.sls.subato.courses.course.Course;
import de.hsrm.sls.subato.courses.course.CourseService;
import de.hsrm.sls.subato.exercises.exercise.ExerciseFilter;
import de.hsrm.sls.subato.exercises.exercise.ExerciseService;
import de.hsrm.sls.subato.exercises.submission.SubmissionService;
import de.hsrm.sls.subato.exercises.taskinstance.TaskInstanceFilter;
import de.hsrm.sls.subato.exercises.taskinstance.TaskInstanceService;
import de.hsrm.sls.subato.solutions.solution.Solution;
import de.hsrm.sls.subato.user.User;
import de.hsrm.sls.subato.user.UserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.stream.Collectors;

@Service
public class CourseResultService {
    @Autowired
    private SubmissionService submissionService;

    @Autowired
    private UserService userService;

    @Autowired
    private ExerciseService exerciseService;

    @Autowired
    private TaskInstanceService taskInstanceService;

    @Autowired
    private CourseService courseService;

    public CourseResult generateFor(long courseId, long userId, boolean hideInvisibleExercises) {
        var course = courseService.findById(courseId);
        var user = userService.findById(userId);
        return generateFor(course, user, hideInvisibleExercises);
    }

    private CourseResult generateFor(Course course, User user, boolean hideInvisibleExercises) {
        var exerciseResults = new ArrayList<ExerciseResult>();
        var exercises = exerciseService.findAll(new ExerciseFilter().courseId(course.getId()), ExercisesAction.EXERCISE_VIEW);

        for(var exercise : exercises) {
            if(hideInvisibleExercises && !exercise.isVisible()) {
                continue;
            }

            var exerciseResult = new ExerciseResult();
            exerciseResult.setExercise(exercise);

            var taskInstances = taskInstanceService.findAll(new TaskInstanceFilter().exerciseId(exercise.getId()));
            exerciseResult.setTaskInstances(taskInstances);

            var submission = submissionService.findByExerciseAndUser(exercise.getId(), user.getId());
            if(submission != null) {
                exerciseResult.setSubmission(submission);

                var solutionsToConsider = submission.getSolution().stream().toList();
                var taskInstanceSolutions = solutionsToConsider.stream().filter(s -> s.getTaskInstance() != null).collect(Collectors.groupingBy(s -> s.getTaskInstance()));
                var latestSolutions = taskInstanceSolutions.entrySet().stream().map(entry -> getLatestSolution(entry.getValue())).collect(Collectors.toList());
                exerciseResult.setSolutions(latestSolutions);

                var testsTotal = latestSolutions.stream().filter(s -> s.getEvalResult() != null).map(s -> s.getEvalResult().getTestsRun()).reduce(0, (a, b) -> a + b);
                var testsPassed = latestSolutions.stream().filter(s -> s.getEvalResult() != null).map(s -> s.getEvalResult().getTestsPassed()).reduce(0, (a, b) -> a + b);
                var leaks = latestSolutions.stream().filter(s -> s.getEvalResult() != null).map(s -> s.getEvalResult().getLeaks()).reduce(0, (a, b) -> a + b);
                var styleErrors = latestSolutions.stream().filter(s -> s.getEvalResult() != null).map(s -> s.getEvalResult().getStyleCheckResults().size()).reduce(0, (a, b) -> a + b);

                exerciseResult.setTestsTotal(testsTotal);
                exerciseResult.setTestsPassed(testsPassed);
                exerciseResult.setLeaksTotal(leaks);
                exerciseResult.setStyleErrorsTotal(styleErrors);
            }

            exerciseResults.add(exerciseResult);
        }
        exerciseResults.sort(Comparator.comparing((ExerciseResult exerciseResult) -> exerciseResult.getExercise().getId()).reversed());
        return new CourseResult(exerciseResults);
    }

    private Solution getLatestSolution(List<Solution> solutions) {
        solutions.sort(Comparator.comparing(Solution::getSubmittedAt).reversed());
        return solutions.stream().findFirst().orElse(null);
    }
}
