package de.hsrm.sls.subato.exercises.submission;

import de.hsrm.sls.subato.auth.principal.SubatoPrincipal;
import de.hsrm.sls.subato.auth.policy.CheckPolicyRequest;
import de.hsrm.sls.subato.auth.policy.PolicyEvaluator;
import de.hsrm.sls.subato.exercises.exercise.Exercise;
import de.hsrm.sls.subato.solutions.solution.Solution;
import de.hsrm.sls.subato.tasks.task.Task;
import de.hsrm.sls.subato.user.User;
import de.hsrm.sls.subato.user.UserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;


@Service
public class SubmissionService {
    @Autowired
    private SubmissionRepository submissionRepository;

    @Autowired
    private UserService userService;

    @Autowired
    private PolicyEvaluator evaluator;

    public Submission findById(long id) {
        return submissionRepository.findById(id).orElseThrow(() -> new SubmissionNotFoundException(id));
    }

    public List<Submission> findAll(SubmissionFilter filter) {
        return findAll(filter, null);
    }

    public List<Submission> findAll(SubmissionFilter filter, String permission) {
        Specification<Submission> spec = filter.toSpecification();

        if (permission != null) {
            var authSpec = evaluator.checkAsSpec(CheckPolicyRequest.fromPrincipal(null, permission));
            spec = spec.and(authSpec);
        }

        var submission = submissionRepository.findAll(spec);
        return submission;
    }

    public Submission findByExerciseAndUser(long exerciseId, long userId) {
        var filter = new SubmissionFilter().submittedById(userId).exerciseId(exerciseId);
        var submissions = findAll(filter);
        return submissions.stream().findFirst().orElse(null);
    }

    public void processSolution(Solution solution) {
        var submission = findByExerciseAndUser(solution.getExercise().getId(), solution.getSubmittedBy().getId());
        if (submission == null) {
            submission = new Submission();
            submission.setExercise(solution.getExercise());
            submission.setSubmittedBy(solution.getSubmittedBy());
        }

        submission.getSolution().add(solution);
        submission = submissionRepository.save(submission);
        solution.setSubmission(submission);
    }

    public int getCurrentAttempts(Task task, Exercise exercise, User user) {
        var submission = findByExerciseAndUser(exercise.getId(), user.getId());
        var attempts = submission != null ? submission.getAttempts() : new ArrayList<TaskInstanceAttempts>();
        return attempts.stream()
                .filter(a -> a.getTaskInstance().getExercise().getId() == exercise.getId() && a.getTaskInstance().getTask().getId() == task.getId())
                .map(a -> a.getAttempts())
                .findFirst()
                .orElse(0);
    }

    public Submission updatePassed(Submission submission, UpdatePassedDto dto, SubatoPrincipal principal) {
        submission.setPassed(dto.getPassed());

        var user = userService.findByPrincipal(principal);
        submission.setCheckedBy(user);

        return submissionRepository.save(submission);
    }

    public Submission updatePoints(Submission submission, UpdatePointsDto dto) {
        var submissionPointRating = submission.getPointRatings().stream()
                .filter(r -> r.getTaskInstance().getTask().getId() == dto.getTaskId())
                .findFirst()
                .orElse(null);

        if (submissionPointRating == null) {
            var taskInstance = submission.getExercise().getTaskInstances().stream()
                    .filter(ti -> ti.getTask().getId() == dto.getTaskId())
                    .findFirst()
                    .get();

            submissionPointRating = new SubmissionPointRating();
            submissionPointRating.setSubmission(submission);
            submissionPointRating.setTaskInstance(taskInstance);
            submission.getPointRatings().add(submissionPointRating);
        }

        submissionPointRating.setPoints(dto.getPoints());

        return submissionRepository.save(submission);
    }
}
