package de.hsrm.sls.subato.shared.config;

import io.swagger.v3.oas.models.Components;
import io.swagger.v3.oas.models.ExternalDocumentation;
import io.swagger.v3.oas.models.OpenAPI;
import io.swagger.v3.oas.models.info.Contact;
import io.swagger.v3.oas.models.info.Info;
import io.swagger.v3.oas.models.security.SecurityRequirement;
import io.swagger.v3.oas.models.security.SecurityScheme;
import io.swagger.v3.oas.models.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.io.ResourceLoader;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.converter.json.MappingJackson2HttpMessageConverter;
import org.springframework.web.servlet.config.annotation.ViewControllerRegistry;
import org.springframework.web.servlet.config.annotation.WebMvcConfigurer;

import java.io.*;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

@Configuration
public class SwaggerConfiguration implements WebMvcConfigurer {
    private final String API_VERSION = "2.0.0-beta15";

    @Value("${keycloak.auth-server-url}")
    private String keycloakUrl;

    @Value("${keycloak.realm}")
    private String keycloakRealm;

    @Value("${subato.swagger.docs_url}")
    private String docsUrl;

    @Value("${subato.swagger.website_url}")
    private String websiteUrl;

    @Value("${subato.swagger.mail}")
    private String mail;

    @Autowired
    ResourceLoader resourceLoader;

    // https://github.com/swagger-api/swagger-ui/issues/6462
    public SwaggerConfiguration(MappingJackson2HttpMessageConverter converter) {
        var supportedMediaTypes = new ArrayList<>(converter.getSupportedMediaTypes());
        supportedMediaTypes.add(new MediaType("application", "octet-stream"));
        converter.setSupportedMediaTypes(supportedMediaTypes);
    }

    private String getApiDescription() {
        try {
            var inputStream = resourceLoader.getResource("classpath:static/api-description.md").getInputStream();
            return new BufferedReader(new InputStreamReader(inputStream))
                    .lines()
                    .collect(Collectors.joining(System.lineSeparator()));
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    @Bean
    public OpenAPI subatoOpenApi() {
        return new OpenAPI()
                .info(new Info().title("Subato - API Dokumentation")
                        .description(getApiDescription())
                        .version(API_VERSION)
                        .contact(new Contact().name("Subato Learning System").url(websiteUrl).email(mail)))
                .externalDocs(new ExternalDocumentation()
                        .description("Subato Dokumentation")
                        .url(docsUrl))
                .tags(List.of(
                        new Tag().name("Kursverwaltung").description("Abfrage und Verwaltung von Kursen."),
                        new Tag().name("Anwesenheitserfassung").description("Abfrage und Verwaltung der Anwesenheit in Kursen."),
                        new Tag().name("Übungsverwaltung").description("Abfrage und Verwaltung von Übungen mit ihren konfigurierten Aufgaben (Aufgabeninstanzen) und Verwaltung von Abgaben/Bewertung"),
                        new Tag().name("Lösungsverwaltung").description("Abgabe, Verwaltung und automatische Auswertung von Lösungen"),
                        new Tag().name("Modulverwaltung").description("Abfrage von Studiengängen, Modulen und anderen Stammdaten wie dem Semester."),
                        new Tag().name("Aufgabenverwaltung").description("Abfrage und Verwaltung von Aufgaben bzw. Aufgaben-Pools"),
                        new Tag().name("Querschnitt").description("Dashboard, Suche, Authentifizierung usw."),
                        new Tag().name("Fehlvorstellungen").description("Intern (Forschung)")
                ))
                .addSecurityItem(new SecurityRequirement().addList("OIDC"))
                .components(
                        new Components()
                                .addSecuritySchemes("OIDC",
                                        new SecurityScheme()
                                                .name("OIDC")
                                                .type(SecurityScheme.Type.OPENIDCONNECT)
                                                .openIdConnectUrl("%s/realms/%s/.well-known/openid-configuration".formatted(keycloakUrl, keycloakRealm))
                                )
                );
    }

    @Override
    public void addViewControllers(ViewControllerRegistry registry) {
        registry.addRedirectViewController("/swagger-ui", "/swagger-ui.html")
                .setKeepQueryParams(true)
                .setStatusCode(HttpStatus.PERMANENT_REDIRECT);
        registry.addRedirectViewController("/swagger-ui/", "/swagger-ui.html")
                .setKeepQueryParams(true)
                .setStatusCode(HttpStatus.PERMANENT_REDIRECT);
    }
}
