package de.hsrm.sls.subato.solutions.evaluation.request;

import de.hsrm.sls.subato.solutions.solution.Solution;
import de.hsrm.sls.subato.solutions.solution.SolutionContentRepository;
import de.hsrm.sls.subato.solutions.evaluation.InvalidSolutionException;
import de.hsrm.sls.subato.shared.fs.FileTree;
import de.hsrm.sls.subato.tasks.stef.spec.FileRef;
import de.hsrm.sls.subato.tasks.stef.spec.FileType;
import de.hsrm.sls.subato.tasks.task.TaskService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;

@Service
public class EvalRequestGenerator {
    @Autowired
    private TaskService taskService;

    @Autowired
    private SolutionContentRepository solutionContentRepository;

    /**
     * resolves the file in the solution, which best matches the template
     * @throws InvalidSolutionException if no file was matched
     */
    private FileTree getMatchingSolutionFile(Solution solution, FileRef templateFile) {
        var solutionTree = solutionContentRepository.getTree(solution);

        var matchers = new FileMatcher[] {
                new FirstFileMatcher(),
                new ExactMatcher(),
                new PrefixMatcher("src"),
                new PrefixMatcher("*"),
                new PrefixMatcher("*/src"),
        };

        FileTree codeFile = null;
        for(var matcher : matchers) {
            var item = matcher.match(solutionTree, templateFile);
            if(item != null) {
                codeFile = item;
                break;
            }
        }

        if(codeFile == null) {
            throw new InvalidSolutionException("Eine Lösungsdatei für %s war erforderlich, wurde aber nicht gefunden".formatted(templateFile.getPath()));
        }

        return codeFile;
    }

    /**
     * selects the files to send to the evaluator and configures other parameters
     * @throws InvalidSolutionException
     */
    public EvalRequest generateRequest(Solution solution, boolean acceptanceCheck) {
        var task = solution.getTask();
        var files = taskService.getFiles(task);

        var filesToTransmit = new ArrayList<FileRef>();
        for(var taskFile : files) {
            var shouldInclude = taskFile.getType() == FileType.INCLUDE || (!acceptanceCheck && taskFile.getType() == FileType.TEST) || (acceptanceCheck && taskFile.getType() == FileType.ACCEPTANCETEST);

            if(shouldInclude) {
                filesToTransmit.add(taskFile);
            }
        }

        var templateFile = files.stream().filter(f -> f.getType() == FileType.TEMPLATE).findFirst().get();
        FileTree codeFile = getMatchingSolutionFile(solution, templateFile);


        var request = new EvalRequest();
        request.setCodeFile(codeFile);
        request.setFilesToTransmit(filesToTransmit);
        request.setTemplateFile(templateFile);

        return request;
    }
}
