package de.hsrm.sls.subato.tasks.stef.parser;

import de.hsrm.sls.subato.shared.markdown.MarkdownParser;
import de.hsrm.sls.subato.tasks.stef.spec.StefSpec;
import de.hsrm.sls.subato.tasks.stef.parser.meta.MetaParser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Path;

/**
 * Responsible for parsing and validating a STEF specification
 */
@Service
public class SpecParser {
    @Autowired
    MetaParser metaParser;

    @Autowired
    MarkdownParser markdownParser;

    /**
     * parses and validates a STEF spec
     * @param fileRoot points to the spec in the file system
     * @return parsed and validated spec
     * @throws InvalidSpecException if the spec is syntactically or semantically invalid
     */
    public StefSpec parseSpec(String fileRoot) {
        var spec = new StefSpec();

        var meta = metaParser.parse(fileRoot);
        spec.setMeta(meta);

        var text = getText(fileRoot);
        spec.setText(text);

        return spec;
    }

    /**
     * reads the task text from the text.html file
     * @param fileRoot path to the STEF spec
     * @return text (in html)
     */
    private String getText(String fileRoot) {
        var textDir = Path.of(fileRoot).resolve("text").toFile();
        if(!textDir.isDirectory()) {
            throw new InvalidSpecException("Der Ordner text existiert nicht, ist jedoch erforderlich");
        }

        var htmlFile = textDir.toPath().resolve("text.html").toFile();
        var mdFile = textDir.toPath().resolve("text.md").toFile();


        if(!htmlFile.exists() && !mdFile.exists()) {
            throw new InvalidSpecException("Eine Aufgabenstellung (text.html oder text.md) existiert nicht, ist jedoch erforderlich");
        }

        if(htmlFile.exists() && mdFile.exists()) {
            throw new InvalidSpecException("Es wurde eine text.html und eine text.md für die Aufgabenstellung angegeben. Es ist nicht klar, welche genommen werden soll.");
        }

        var file = htmlFile.exists() ? htmlFile : mdFile;

        try {
            String content = Files.readString(file.toPath(), StandardCharsets.UTF_8);
            if(file.equals(mdFile)) {
                content = markdownParser.convertToHtml(content);
            }

            if (file.equals(htmlFile)) {
                var sanitizer = new TexSanitizer();
                content = sanitizer.removeHtmlInTexExpressions(content);
            }

            return content;
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }
}
