package de.hsrm.sls.subato.tasks.task;

import de.hsrm.sls.subato.shared.config.SubatoConfig;
import de.hsrm.sls.subato.shared.fs.TmpDirProvider;
import de.hsrm.sls.subato.shared.helper.FileHelper;
import de.hsrm.sls.subato.shared.helper.ZipHelper;
import de.hsrm.sls.subato.tasks.imp.TaskImportException;
import de.hsrm.sls.subato.tasks.stef.spec.FileRef;
import de.hsrm.sls.subato.tasks.stef.spec.FileType;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.FileSystemUtils;

import java.io.File;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.ArrayList;

@Service
public class BundleService {
    @Autowired
    private TmpDirProvider tmpDirProvider;

    @Autowired
    private TaskContentRepository taskContentRepository;

    @Autowired
    private SubatoConfig subatoConfig;

    private File getBundleFile(Task task) {
        return Paths.get(subatoConfig.getDataDir())
                .resolve("bundle")
                .resolve(String.valueOf(task.getId()))
                .resolve("%s.zip".formatted(task.getIdName()))
                .toFile();
    }

    public File getBundle(Task task) {
        if(task.isArchived()) {
            throw new TaskArchivedException();
        }

        return getBundleFile(task);
    }

    /**
     * generates a zip file, bundling all public source files (excluding the solution) associated with the given task.
     * then stores the zip file in the file system
     *
     * useful for students wanting to import the task into their IDE without explicitly downloading each file manually
     *
     */
    public File buildBundle(Task task) {
        var taskPath = taskContentRepository.getTaskPath(task);
        var spec = task.getSpec();

        var srcFiles = spec.getMeta().getFiles().stream()
                .filter(fileRef -> fileRef.isPublic())
                .filter(fileRef -> fileRef.getType() == FileType.TEMPLATE ||
                        fileRef.getType() == FileType.INCLUDE ||
                        fileRef.getType() == FileType.TEST)
                .toList();

        var miscFiles = spec.getMeta().getFiles().stream()
                .filter(fileRef -> fileRef.isPublic())
                .filter(fileRef -> fileRef.getType() == FileType.MISC)
                .toList();
        var targetFiles = new ArrayList<FileRef>();
        targetFiles.addAll(srcFiles);
        targetFiles.addAll(miscFiles);

        if(targetFiles.size() == 0) {
            return null;
        }

        var tempDir = tmpDirProvider.createTmpPath().toFile();

        try {
            for (var fileRef : targetFiles) {
                try {
                    var targetPath = srcFiles.contains(fileRef)
                            ? tempDir.toPath().resolve("src").resolve(Path.of(fileRef.getPath()))
                            : tempDir.toPath().resolve("resources").resolve(Path.of(fileRef.getPath()));
                    var targetDirectory = targetPath.getParent();
                    Files.createDirectories(targetDirectory);

                    var bytes = FileHelper.toBytes(fileRef.getFile().getFile(taskPath));
                    Files.write(targetPath, bytes);
                } catch (IOException e) {
                    throw new TaskImportException("bundle could not be built", e);
                }
            }

            var bundleFile = getBundleFile(task);
            bundleFile.mkdirs();
            return ZipHelper.zip(tempDir, bundleFile, false);
        } finally {
            FileSystemUtils.deleteRecursively(tempDir);
        }
    }
}
