package de.hsrm.sls.subato.tasks.task;

import de.hsrm.sls.subato.solutions.solution.Solution;
import de.hsrm.sls.subato.solutions.evaluation.evaluator.Evaluator;
import de.hsrm.sls.subato.tasks.feedback.TaskFeedback;
import de.hsrm.sls.subato.tasks.feedback.TaskFeedbackStats;
import de.hsrm.sls.subato.tasks.lang.Language;
import de.hsrm.sls.subato.exercises.taskinstance.TaskInstance;
import de.hsrm.sls.subato.tasks.pool.TaskPool;
import de.hsrm.sls.subato.tasks.stef.spec.*;

import de.hsrm.sls.subato.tasks.stef.cache.StefSpecCache;
import jakarta.persistence.*;
import org.hibernate.annotations.JdbcTypeCode;
import org.hibernate.type.SqlTypes;

import java.nio.file.Path;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Objects;

import static de.hsrm.sls.subato.shared.helper.UriHelper.concat;

/**
 * represents a task (STEF) in the database (used for linking to other entities)
 */
@Entity
@Table(name = "task", uniqueConstraints={
        // task ids are unique within a task pool
        @UniqueConstraint(columnNames = {"idName", "pool_id"})
})
public class Task {
    /**
     * unique across all tasks, just for internal use
     */
    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    private long id;

    /**
     * STEF
     */
    private String idName;

    /**
     * STEF
     */
    private String name;

    /**
     * actual occurences of this task in exercises
     */
    @OneToMany(mappedBy = "task", cascade = CascadeType.REMOVE)
    private List<TaskInstance> instances;

    @ManyToOne
    private TaskPool pool;

    /**
     * STEF
     * maps to subatoId
     */
    @Column(unique = true)
    private Long sourceId;

    /**
     * STEF
     * just for display purposes
     */
    @ManyToOne
    private Language language;

    /**
     * STEF
     * null if solutions are not evaluated automatically
     */
    @ManyToOne(cascade = { CascadeType.PERSIST, CascadeType.MERGE, CascadeType.REFRESH })
    private Evaluator evaluator;


    /**
     * feedback entries for this task
     */
    @OneToMany(mappedBy = "task", cascade = CascadeType.ALL, orphanRemoval = true)
    private List<TaskFeedback> feedback = new ArrayList<>();

    /**
     * flag for soft deletion.
     *
     * soft deleted tasks are preserved in order to keep references from solutions.
     * useful if someone deletes a task accidentally.
     *
     * archived tasks may have an invalid or missing spec.
     */
    private boolean archived;

    @OneToMany(mappedBy = "task", cascade = CascadeType.REMOVE)
    private List<Solution> solutions;

    /**
     * relative url to the task in the repository
     * only applicable if the linked task pool is linked to a git repository
     */
    private String repositoryPath;

    @JdbcTypeCode(SqlTypes.JSON)
    private StefSpecCache specCache;

    private String bundle;

    public Task() { }

    public Long getSourceId() {
        return sourceId;
    }

    public void setSourceId(Long sourceId) {
        this.sourceId = sourceId;
    }

    public long getId() {
        return id;
    }

    public void setId(long id) {
        this.id = id;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public List<TaskInstance> getInstances() {
        return instances;
    }

    public void setInstances(List<TaskInstance> instances) {
        this.instances = instances;
    }

    public TaskPool getPool() {
        return pool;
    }

    public void setPool(TaskPool pool) {
        this.pool = pool;
    }

    public Language getLanguage() {
        return language;
    }

    public void setLanguage(Language language) {
        this.language = language;
    }

    public Evaluator getEvaluator() {
        return evaluator;
    }

    public void setEvaluator(Evaluator evaluator) {
        this.evaluator = evaluator;
    }

    public String getIdName() {
        return idName;
    }

    public void setIdName(String idName) {
        this.idName = idName;
    }

    public List<TaskFeedback> getFeedback() {
        return feedback;
    }

    public void setFeedback(List<TaskFeedback> feedback) {
        this.feedback = feedback;
    }

    public SubmissionModeType getDefaultSubmissionMode() {
        var spec = getSpec();
        return spec != null ? spec.getMeta().getSubmissionModeConfig().getDefaultMode() : null;
    }

    public boolean isArchived() {
        return archived;
    }

    public void setArchived(boolean archived) {
        this.archived = archived;
    }

    public List<Solution> getSolutions() {
        return solutions;
    }

    public void setSolutions(List<Solution> solutions) {
        this.solutions = solutions;
    }

    /**
     * calculates the average of complexity, comprehensibility feedback entries
     * and counts the number of unchecked entries
     */
    public TaskFeedbackStats getFeedbackStats() {
        if(feedback.stream().count() == 0) {
            return null;
        }

        var unread = feedback.stream().filter(f -> !f.isRead()).count();
        var avgComplexity = feedback.stream().map(f -> f.getComplexity()).reduce(0, (sum, c) -> sum + c);
        var avgComprehensibility = feedback.stream().map(f -> f.getComprehensibility()).reduce(0, (sum, c) -> sum + c);

        var stats = new TaskFeedbackStats();
        stats.setUnread(unread);
        stats.setAvgComplexity(avgComplexity);
        stats.setAvgComprehensibility(avgComprehensibility);
        return stats;
    }

    /**
     * determines if lecturers can run additional acceptance tests while checking/grading the solution
     */
    public boolean isAcceptanceTestAvailable() {
        var spec = getSpec();
        return spec != null &&  spec.getMeta().getEvaluator() != null && spec.getMeta().getFiles().stream().anyMatch(f -> f.getType() == FileType.ACCEPTANCETEST);
    }

    public boolean isSupportsFileSubmissionMode() {
        return getFileSubmissionMode() != null;
    }

    public boolean isSupportsTextSubmissionMode() {
        return getTextSubmissionMode() != null;
    }

    /**
     * determines if solutions for this task can be evaluated automatically
     */
    public boolean evaluationPossible() {
        var spec = getSpec();
        return spec != null && spec.getMeta().getEvaluator() != null && spec.getMeta().getFiles().stream().anyMatch(f -> f.getType() == FileType.TEST);
    }

    public String getRepositoryPath() {
        return repositoryPath;
    }

    public void setRepositoryPath(String repositoryPath) {
        this.repositoryPath = repositoryPath;
    }

    public boolean isSupportedByEclipsePlugin() {
        var supportedLanguages = new String[] { Language.JAVA };
        return !archived &&
                language != null &&
                Arrays.stream(supportedLanguages).anyMatch(langId -> language.getId().equals(langId)) &&
                isAllowArchives();
    }

    public StefSpecCache getSpecCache() {
        return specCache;
    }

    public StefSpec getSpec() {
        return hasValidSpec() ? specCache.getSpec() : null;
    }

    public void setSpecCache(StefSpecCache specCache) {
        this.specCache = specCache;
    }

    public String getRepositoryUrl() {
        if(pool.isLinkedToRepository() && repositoryPath != null) {
            return concat(pool.getRepositoryUrl(), Path.of("-").resolve("tree").resolve("main").resolve(repositoryPath));
        }

        return null;
    }

    public boolean isAllowArchives() {
        var fileSubmissionMode = getFileSubmissionMode();
        return fileSubmissionMode != null && fileSubmissionMode.isAllowArchives();
    }

    public List<String> getAllowedExtensions() {
        var fileSubmissionMode = getFileSubmissionMode();
        if(fileSubmissionMode == null) {
            return new ArrayList<>();
        }

        var allowedExtensions = fileSubmissionMode.getExtensions();
        if(isAllowArchives()) {
            allowedExtensions.addAll(SupportedArchiveExtensions.getSupportedExtensions());
        }

        return allowedExtensions;
    }

    public FileSubmissionMode getFileSubmissionMode() {
        var spec = getSpec();
        if(spec == null) {
            return null;
        }

        return spec.getMeta().getSubmissionModeConfig().getFileSubmissionMode();
    }

    public TextSubmissionMode getTextSubmissionMode() {
        var spec = getSpec();
        if(spec == null) {
            return null;
        }

        return spec.getMeta().getSubmissionModeConfig().getTextSubmissionMode();
    }

    public boolean isExtractArchives() {
        var fileSubmissionMode = getFileSubmissionMode();
        return fileSubmissionMode != null && fileSubmissionMode.isExtractArchives();
    }

    public boolean hasValidSpec() {
        return specCache != null;
    }

    public String getBundle() {
        return bundle;
    }

    public void setBundle(String bundle) {
        this.bundle = bundle;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        Task task = (Task) o;
        return id == task.id;
    }

    @Override
    public int hashCode() {
        return Objects.hash(id);
    }

    @PreRemove
    private void detachEvaluator() {
        evaluator = null;
    }
}
